# Copyright (c) 2004-2007 Bethel University. All rights reserved.
# See also gpl.txt
# Zope
from zope.interface import implements
from Globals import InitializeClass
from persistent.list import PersistentList
from persistent.mapping import PersistentMapping
from AccessControl import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from OFS.SimpleItem import SimpleItem
from OFS.Cache import Cacheable
from AccessControl import getSecurityManager
from DateTime import DateTime

# Silva
from Products.Silva import SilvaPermissions
from Products.Silva import mangle
from Products.Silva.Asset import Asset
from Products.Silva.helpers import add_and_edit
from Products.Silva.adapters.virtualhosting import getVirtualHostingAdapter

from interfaces import IStylesheet

"""Stylesheet uses formulator forms to display the various css declarations in Silva.
Some of these css declarations contain characters that can't be part of name and id attributes.  Example:
div.citation p
a:hover
.margins
' ' and ':' are invalid characters.  To get around this, the id's of the formulator fields containing these are modified, such that:
' ' = ..
':' = __
^\. = zz.

When the stylesheet is rendered, the '..' and '__' are replaced with ' ' and ':'.  The process is reveresed when saving the stylesheets.

To customize this for your environment, there is one value in Stylesheet that needs to be updated.  In __init__, self._css_prefix needs to be updated.  This value is prefixed to every style in the stylesheet.  E.g.  The default '#bu-content-pad' is prefixed to every style, like:
#bu-content-pad strong { ... }
"""

class Stylesheet(Asset, SimpleItem, Cacheable):
    __doc__ = "A Stylesheet Asset that is editable in silva.  It allows you to customize the various aspects of Silva Document content"

    security = ClassSecurityInfo()

    meta_type = "Silva Stylesheet"

    implements(IStylesheet)

    manage_options = Cacheable.manage_options

    def __init__(self, id):
        Stylesheet.inheritedAttribute('__init__')(self, id)
        #may will be used for caching
        self._previous_lastmod = DateTime()
        self._cached_css = ''
        self._css = PersistentMapping()
        self._css_prefix = '#bu-content-pad'
        self._inherit_styles = False

    # ACCESSORS
    security.declareProtected(SilvaPermissions.View, 'is_cacheable')
    def is_cacheable(self):
        #could be cacheable, if the modified time is compared to a last modified time...
        return 1

    security.declareProtected(SilvaPermissions.View, 'inherit_styles')
    def inherit_styles(self):
        return self._inherit_styles

    security.declareProtected(SilvaPermissions.ChangeSilvaContent, 'set_inherit_styles')
    def set_inherit_styles(self, decision):
        self._inherit_styles = not not decision

    #will raise an attribute error if a style contains invalid characters.
    #currently this is just '}'.
    security.declareProtected(SilvaPermissions.ChangeSilvaContent, 'save_styles')
    def save_styles(self, set, styles):
        #set is the css set (e.g. paragraph, heading, anchor, etc)
        #styles is a formulator results hash.
        if not hasattr(self._css,set):
            self._css[set] = PersistentMapping()
        css_set = self._css[set]
        for (style,decls) in styles.items():
            for d in decls:
                if set!='custom' and d.find('}') > -1:
                    raise AttributeError('Invalid character "}" in style declaration.')
            css_set[style] = PersistentList(decls)
        self.ZCacheable_invalidate()

    security.declareProtected(SilvaPermissions.ChangeSilvaContent, 'save_css_prefix')
    def save_css_prefix(self, prefix):
        #css prefix is appended to every style decl, like 'css_prefix a:link {}'
        self._css_prefix = str(prefix).strip()
        self.ZCacheable_invalidate()

    security.declareProtected(SilvaPermissions.AccessContentsInformation, 'get_css_prefix')
    def get_css_prefix(self):
        return self._css_prefix

    #returns a list containing set['styles'].
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'get_style')
    def get_style(self, set, style):
        return '\n'.join(self._css.get(set,{}).get(style,[]))
            
InitializeClass(Stylesheet)
