# Copyright (c) 2004-2007 Bethel University. All rights reserved.
# See also gpl.txt

from AccessControl import getSecurityManager

from Products.Five import BrowserView
from Products.Silva.adapters.virtualhosting import getVirtualHostingAdapter
from Products.SilvaStylesheets.interfaces import IStylesheet

from zLOG import LOG,INFO

class StylesheetRenderer(BrowserView):
    def render(self):
        request = self.context.REQUEST
        response = request.RESPONSE

        response.setHeader('Content-Type','text/css; charset=utf8')
        keyset = None
        #prepare keyset and see if result is in cache
        if self.context.ZCacheable_isCachingEnabled():
            adapter = getVirtualHostingAdapter(self.context)
            # Prepare a cache key.
            keyset = {'public': adapter.getVirtualHostKey(),
                      'user': getSecurityManager().getUser()}
            result = self.context.ZCacheable_get(keywords=keyset)
            #AccelHTTPCacheMgr always returns None (i.e. default)
            if result is not None:
                return result

        #chech to see if content has been modified lately.
        if self.context.bobobase_modification_time() > self.context._previous_lastmod:
            self.context._previous_lastmod = self.context.bobobase_modification_time()
            self.context._cached_css = self.render_css()

        if keyset is not None:
            self.context.ZCacheable_set(self.context._cached_css, keywords=keyset)
        return self.context._cached_css

    def render_css(self, linebreak='\n'):
        css = self.get_parent_styles()
        for set in self.context._css.values():
            for (style,decls) in set.items():
                if decls:
                    css.append(self.render_one_style(style,decls,linebreak))
        return linebreak.join(css)

    def get_parent_styles(self):
        parent_styles = []
        css = []
        if self.context.inherit_styles():
            adapter = getVirtualHostingAdapter(self.context)
            #virtualhost root, or silva root
            vr = adapter.getVirtualRoot() or self.context.get_root()
            vrp = vr.aq_parent
            #self.context is the IStylesheet, so need to 2-up it
            cont = self.context.get_container().get_real_container()
            while (cont and cont != vrp):
                ls = getattr(cont.aq_explicit,'local-styles.css',None)
                if ls:
                    if IStylesheet.providedBy(ls):
                        parent_styles.append(ls)
                        if not ls.inherit_styles():
                            break
                cont = cont.aq_parent
            if parent_styles:
                parent_styles.reverse()
                for p in parent_styles:
                    css.append('@import url("%s");'%(p.absolute_url()))
        return css
        

    def render_one_style(self, style, decls, linebreak):
        name = self.unmangle_style_name(style)
        if self.context._css_prefix:
            name = '%s %s'%(self.context._css_prefix,name)
        if style=='custom':
            decls = ''.join(decls).split('}')
            decls = [ "%s %s}"%(self.context._css_prefix,d) for d in decls if d]
            return linebreak.join(decls)
        return '%s {\n %s\n}'%(name,linebreak.join(decls))
    
    def unmangle_style_name(self, style):
        return style.replace('__',':').replace('..',' ').replace('zz.','.')
